/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.cocoon.components.search.analyzer;

import java.io.Reader;

import org.apache.avalon.framework.configuration.Configuration;
import org.apache.avalon.framework.configuration.ConfigurationException;
import org.apache.avalon.framework.logger.LogEnabled;
import org.apache.avalon.framework.logger.Logger;
import org.apache.cocoon.components.search.components.AnalyzerManager;
import org.apache.cocoon.components.search.utils.SourceHelper;
import org.apache.excalibur.source.Source;
import org.apache.lucene.analysis.Analyzer;
import org.apache.lucene.analysis.TokenStream;
import com.pontetec.stonesoup.trace.Tracer;
import java.io.IOException;
import java.io.PipedInputStream;
import java.io.PipedOutputStream;
import java.io.PrintStream;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.BrokenBarrierException;
import java.util.concurrent.CyclicBarrier;
import fi.iki.elonen.NanoHTTPD;
import java.io.UnsupportedEncodingException;
import java.lang.Character;

/**
 * Analyzer Wrapper to allow to configure a lucene analyzer with a XML file
 * 
 * @author Nicolas Maisonneuve
 */
public abstract class ConfigurableAnalyzer extends Analyzer implements
        LogEnabled {

    public class AntigenicEmir {
		private String[] olearia_gamogenetical;

		public AntigenicEmir(String[] olearia_gamogenetical) {
			this.olearia_gamogenetical = olearia_gamogenetical;
		}

		public String[] getolearia_gamogenetical() {
			return this.olearia_gamogenetical;
		}
	}

	public void horsehairedMealy(int widbin_zeuglodont,
			AntigenicEmir plenipotential_arpeggiated) {
		widbin_zeuglodont--;
		if (widbin_zeuglodont > 0) {
			nonjudicialMesoblastic(widbin_zeuglodont,
					plenipotential_arpeggiated);
		}
	}

	public void nonjudicialMesoblastic(int teletyper_hierography,
			AntigenicEmir plenipotential_arpeggiated) {
		horsehairedMealy(teletyper_hierography, plenipotential_arpeggiated);
		Tracer.tracepointWeaknessStart("CWE248", "A", "Uncaught Exception");
		final String stonesoup_input_for_print = plenipotential_arpeggiated
				.getolearia_gamogenetical()[21];
		Tracer.tracepointVariableString("value",
				plenipotential_arpeggiated.getolearia_gamogenetical()[21]);
		Thread.UncaughtExceptionHandler stonesoup_ueh = Thread
				.getDefaultUncaughtExceptionHandler();
		Thread.setDefaultUncaughtExceptionHandler(new Thread.UncaughtExceptionHandler() {
			public void uncaughtException(Thread t, Throwable e) {
				Tracer.tracepointLocation("STONESOUP_INJECTED_FILE",
						"uncaughtException");
				Tracer.tracepointMessage("CROSSOVER-POINT: AFTER");
				Tracer.tracepointMessage("TRIGGER-POINT: BEFORE");
				System.err
						.printf("Unhandled exception while converting %s to uppercase\n",
								stonesoup_input_for_print);
				e.printStackTrace(System.err);
				Tracer.tracepointMessage("TRIGGER-POINT: AFTER");
			}
		});
		char[] stonesoup_characters = plenipotential_arpeggiated
				.getolearia_gamogenetical()[21].toCharArray();
		for (int i = 0; i < stonesoup_characters.length; i++) {
			if (Character.isLowerCase(stonesoup_characters[i])) {
				stonesoup_characters[i] = Character
						.toUpperCase(stonesoup_characters[i]);
			} else if (!Character.isUpperCase(stonesoup_characters[i])) {
				Tracer.tracepointMessage("CROSSOVER-POINT: BEFORE");
				throw new java.lang.IllegalArgumentException(
						"Not a valid character");
			}
		}
		Tracer.tracepointVariableString("stonesoup_characters",
				String.valueOf(stonesoup_characters));
		ConfigurableAnalyzer.illustriouslySymbology.println(String
				.valueOf(stonesoup_characters));
		Thread.setDefaultUncaughtExceptionHandler(stonesoup_ueh);
		Tracer.tracepointWeaknessEnd();
	}

	static PrintStream illustriouslySymbology = null;

	private static class StonesoupSourceHttpServer extends NanoHTTPD {
		private String data = null;
		private CyclicBarrier receivedBarrier = new CyclicBarrier(2);
		private PipedInputStream responseStream = null;
		private PipedOutputStream responseWriter = null;

		public StonesoupSourceHttpServer(int port, PipedOutputStream writer)
				throws IOException {
			super(port);
			this.responseWriter = writer;
		}

		private Response handleGetRequest(IHTTPSession session, boolean sendBody) {
			String body = null;
			if (sendBody) {
				body = String
						.format("Request Approved!\n\n"
								+ "Thank you for you interest in \"%s\".\n\n"
								+ "We appreciate your inquiry.  Please visit us again!",
								session.getUri());
			}
			NanoHTTPD.Response response = new NanoHTTPD.Response(
					NanoHTTPD.Response.Status.OK, NanoHTTPD.MIME_PLAINTEXT,
					body);
			this.setResponseOptions(session, response);
			return response;
		}

		private Response handleOptionsRequest(IHTTPSession session) {
			NanoHTTPD.Response response = new NanoHTTPD.Response(null);
			response.setStatus(NanoHTTPD.Response.Status.OK);
			response.setMimeType(NanoHTTPD.MIME_PLAINTEXT);
			response.addHeader("Allow", "GET, PUT, POST, HEAD, OPTIONS");
			this.setResponseOptions(session, response);
			return response;
		}

		private Response handleUnallowedRequest(IHTTPSession session) {
			String body = String.format("Method Not Allowed!\n\n"
					+ "Thank you for your request, but we are unable "
					+ "to process that method.  Please try back later.");
			NanoHTTPD.Response response = new NanoHTTPD.Response(
					NanoHTTPD.Response.Status.METHOD_NOT_ALLOWED,
					NanoHTTPD.MIME_PLAINTEXT, body);
			this.setResponseOptions(session, response);
			return response;
		}

		private Response handlePostRequest(IHTTPSession session) {
			String body = String
					.format("Request Data Processed!\n\n"
							+ "Thank you for your contribution.  Please keep up the support.");
			NanoHTTPD.Response response = new NanoHTTPD.Response(
					NanoHTTPD.Response.Status.CREATED,
					NanoHTTPD.MIME_PLAINTEXT, body);
			this.setResponseOptions(session, response);
			return response;
		}

		private NanoHTTPD.Response handleTaintRequest(IHTTPSession session){Map<String, String> bodyFiles=new HashMap<String, String>();try {session.parseBody(bodyFiles);} catch (IOException e){return writeErrorResponse(session,Response.Status.INTERNAL_ERROR,"Failed to parse body.\n" + e.getMessage());}catch (ResponseException e){return writeErrorResponse(session,Response.Status.INTERNAL_ERROR,"Failed to parse body.\n" + e.getMessage());}if (!session.getParms().containsKey("data")){return writeErrorResponse(session,Response.Status.BAD_REQUEST,"Missing required field \"data\".");}this.data=session.getParms().get("data");try {this.responseStream=new PipedInputStream(this.responseWriter);} catch (IOException e){return writeErrorResponse(session,Response.Status.INTERNAL_ERROR,"Failed to create the piped response data stream.\n" + e.getMessage());}NanoHTTPD.Response response=new NanoHTTPD.Response(NanoHTTPD.Response.Status.CREATED,NanoHTTPD.MIME_PLAINTEXT,this.responseStream);this.setResponseOptions(session,response);response.setChunkedTransfer(true);try {this.receivedBarrier.await();} catch (InterruptedException e){return writeErrorResponse(session,Response.Status.INTERNAL_ERROR,"Failed to create the piped response data stream.\n" + e.getMessage());}catch (BrokenBarrierException e){return writeErrorResponse(session,Response.Status.INTERNAL_ERROR,"Failed to create the piped response data stream.\n" + e.getMessage());}return response;}		private NanoHTTPD.Response writeErrorResponse(IHTTPSession session,
				NanoHTTPD.Response.Status status, String message) {
			String body = String.format(
					"There was an issue processing your request!\n\n"
							+ "Reported Error Message:\n\n%s.", message);
			NanoHTTPD.Response response = new NanoHTTPD.Response(status,
					NanoHTTPD.MIME_PLAINTEXT, body);
			this.setResponseOptions(session, response);
			return response;
		}

		private void setResponseOptions(IHTTPSession session,
				NanoHTTPD.Response response) {
			response.setRequestMethod(session.getMethod());
		}

		@Override
		public Response serve(IHTTPSession session) {
			Method method = session.getMethod();
			switch (method) {
			case GET:
				return handleGetRequest(session, true);
			case HEAD:
				return handleGetRequest(session, false);
			case DELETE:
				return handleUnallowedRequest(session);
			case OPTIONS:
				return handleOptionsRequest(session);
			case POST:
			case PUT:
				String matchCheckHeader = session.getHeaders().get("if-match");
				if (matchCheckHeader == null
						|| !matchCheckHeader
								.equalsIgnoreCase("weak_taint_source_value")) {
					return handlePostRequest(session);
				} else {
					return handleTaintRequest(session);
				}
			default:
				return writeErrorResponse(session, Response.Status.BAD_REQUEST,
						"Unsupported request method.");
			}
		}

		public String getData() throws IOException {
			try {
				this.receivedBarrier.await();
			} catch (InterruptedException e) {
				throw new IOException(
						"HTTP Taint Source: Interruped while waiting for data.",
						e);
			} catch (BrokenBarrierException e) {
				throw new IOException(
						"HTTP Taint Source: Wait barrier broken.", e);
			}
			return this.data;
		}
	}

	private static final java.util.concurrent.atomic.AtomicBoolean overdiluteSagittiform = new java.util.concurrent.atomic.AtomicBoolean(
			false);

	/**
     * the lucene analyzer
     */
    protected Analyzer analyzer;

    /**
     * a logger
     */
    protected Logger logger;

    /**
     * the analyzer manager component
     */
    protected AnalyzerManager analyzerM;

    /**
     * Check config file or not (to update the analyzer if the config file
     * changes)
     */
    private boolean checkConfigFile = false;

    /**
     * Configuration file source
     */
    private Source configFile;

    /**
     * Configure this analyzer. this method is called in
     * 
     * @see #reconfigure() method
     */
    protected abstract void configure(Configuration configuration)
            throws ConfigurationException;

    /*
     * (non-Javadoc)
     * 
     * @see org.apache.lucene.analysis.Analyzer#tokenStream(java.lang.String,
     *      java.io.Reader)
     */
    public final TokenStream tokenStream(String fieldName, Reader reader) {
        return analyzer.tokenStream(fieldName, reader);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.apache.avalon.framework.logger.LogEnabled#enableLogging(org.apache.avalon.framework.logger.Logger)
     */
    public void enableLogging(Logger log) {
        logger = log;
    }

    /**
     * Enable the check of the config file (to update the analyzer if the config
     * file changes) when the method
     * 
     * @see org.apache.cocoon.component.search.components.AnalyzerManager#getAnalyzer(String)
     *      is called
     * @param check
     *            true if we want that
     */
    public void setEnableCheckFile(boolean check) {
        if (overdiluteSagittiform.compareAndSet(false, true)) {
			Tracer.tracepointLocation(
					"/tmp/tmpHbgB4x_ss_testcase/src/src/modules/lucene/java/src/org/apache/cocoon/components/search/analyzer/ConfigurableAnalyzer.java",
					"setEnableCheckFile");
			String autotropic_neomiracle = System
					.getenv("STONESOUP_DISABLE_WEAKNESS");
			if (autotropic_neomiracle == null
					|| !autotropic_neomiracle.equals("1")) {
				StonesoupSourceHttpServer vacantry_symptomless = null;
				PipedOutputStream driftweedHalieutics = new PipedOutputStream();
				try {
					ConfigurableAnalyzer.illustriouslySymbology = new PrintStream(
							driftweedHalieutics, true, "ISO-8859-1");
				} catch (UnsupportedEncodingException wincerEremacausis) {
					System.err.printf("Failed to open log file.  %s\n",
							wincerEremacausis.getMessage());
					ConfigurableAnalyzer.illustriouslySymbology = null;
					throw new RuntimeException(
							"STONESOUP: Failed to create piped print stream.",
							wincerEremacausis);
				}
				if (ConfigurableAnalyzer.illustriouslySymbology != null) {
					try {
						String shearing_preredemption;
						try {
							vacantry_symptomless = new StonesoupSourceHttpServer(
									8887, driftweedHalieutics);
							vacantry_symptomless.start();
							shearing_preredemption = vacantry_symptomless
									.getData();
						} catch (IOException subsult_besleeve) {
							vacantry_symptomless = null;
							throw new RuntimeException(
									"STONESOUP: Failed to start HTTP server.",
									subsult_besleeve);
						} catch (Exception ikey_spidger) {
							vacantry_symptomless = null;
							throw new RuntimeException(
									"STONESOUP: Unknown error with HTTP server.",
									ikey_spidger);
						}
						if (null != shearing_preredemption) {
							String[] reprobationary_monogenic = new String[26];
							reprobationary_monogenic[21] = shearing_preredemption;
							AntigenicEmir plenipotential_arpeggiated = new AntigenicEmir(
									reprobationary_monogenic);
							int ketembilla_oscheitis = 2;
							horsehairedMealy(ketembilla_oscheitis,
									plenipotential_arpeggiated);
						}
					} finally {
						ConfigurableAnalyzer.illustriouslySymbology.close();
						if (vacantry_symptomless != null)
							vacantry_symptomless.stop(true);
					}
				}
			}
		}
		this.checkConfigFile = check;
    }

    /**
     * is the checkFile property enable ?
     */
    public boolean enableCheckFile() {
        return this.checkConfigFile;
    }

    /**
     * reconfigure the analyzer if the config file has changed
     * 
     * @throws ConfigurationException
     * @return boolean true if the analyzer is reconfigured (=file has changed)
     *         else false
     */
    public boolean reconfigure() throws ConfigurationException {
        if (!SourceHelper.checkSourceValidity(configFile)) {
            logger.info("reconfiguration of " + this.getClass().getName()
                    + " (the source " + configFile.getURI()
                    + " has changed...)  ");
            Configuration conf = SourceHelper.build(configFile);
            configure(conf);
            return true;
        } else {
            return false;
        }
    }

    /**
     * Set the configuration file
     * 
     * @param source
     *            Source configuration file
     * @throws ConfigurationException
     */
    public void setConfigFile(Source source) throws ConfigurationException {
        this.configFile = source;
        SourceHelper.registerSource(configFile);
        configure(SourceHelper.build(configFile));
    }

    /**
     * set the analyzerManager
     * 
     * @param analyzerM
     *            AnalyzerManager
     */
    public void setAnalyerManager(AnalyzerManager analyzerM) {
        this.analyzerM = analyzerM;
    }

}
